!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------------
! NASA GSFC Land surface Verification Toolkit (LVT) V1.0
!-------------------------END NOTICE -- DO NOT EDIT-----------------------------
!BOP
! 
! !MODULE: ALEXI_obsMod
! \label(ALEXI_obsMod)
!
! !INTERFACE:
module ALEXI_obsMod
! 
! !USES: 
  use ESMF

  implicit none

  PRIVATE 
!
! !INPUT PARAMETERS: 
! 
! !OUTPUT PARAMETERS:
!
! !DESCRIPTION: 
! 
! !FILES USED:
!
! !REVISION HISTORY: 
!  18 May 2011   Sujay Kumar  Initial Specification
! 
!EOP
!

!-----------------------------------------------------------------------------
! !PUBLIC MEMBER FUNCTIONS:
!-----------------------------------------------------------------------------
  PUBLIC :: ALEXI_obsinit !Initializes structures for reading ALEXI data
!-----------------------------------------------------------------------------
! !PUBLIC TYPES:
!-----------------------------------------------------------------------------
  PUBLIC :: ALEXIobs !Object to hold ALEXI observation attributes
!EOP

  type, public :: alexidec
     character*100           :: odir
     integer                 :: nc,nr
     integer                 :: res
     logical                 :: startFlag
     real, allocatable           :: rlat(:)
     real, allocatable           :: rlon(:)
     integer, allocatable        :: n11(:)
  end type alexidec
     
  type(alexidec), save :: ALEXIObs(2)

contains
  
!BOP
! 
! !ROUTINE: ALEXI_obsInit
! \label{ALEXI_obsInit}
!
! !INTERFACE: 
  subroutine ALEXI_obsinit(source)
! 
! !USES: 
    use LVT_coreMod
    use LVT_histDataMod
    use LVT_timeMgrMod
    use LVT_logMod

    implicit none
!
! !INPUT PARAMETERS: 

! 
! !OUTPUT PARAMETERS:
!
! !DESCRIPTION: 
!   This subroutine initializes and sets up the data structures required
!   for reading the ALEXI data, including the computation of spatial 
!   interpolation weights. The ALEXI data is provides in the 
!   EASE grid projection. 
! 
! !FILES USED:
!
! !REVISION HISTORY: 
! 
!EOP
    integer               :: source
    integer               :: status
    real                  :: gridDesci(50)

    call ESMF_ConfigGetAttribute(LVT_Config, ALEXIobs(source)%odir, &
         label='ALEXI data directory: ',rc=status)
    call LVT_verify(status, 'ALEXI data directory: not defined')

    call ESMF_ConfigGetAttribute(LVT_Config, ALEXIobs(source)%res, &
         label='ALEXI data resolution (in km): ',rc=status)
    call LVT_verify(status, 'ALEXI data resolution (in km): not defined')

    call LVT_update_timestep(LVT_rc, 86400)

    allocate(ALEXIobs(source)%rlat(LVT_rc%lnc*LVT_rc%lnr))
    allocate(ALEXIobs(source)%rlon(LVT_rc%lnc*LVT_rc%lnr))

    allocate(ALEXIobs(source)%n11(LVT_rc%lnc*LVT_rc%lnr))

    gridDesci = 0
    
    if(ALEXIobs(source)%res.eq.10) then 
       !filling the items needed by the interpolation library
       gridDesci(1) = 0  
       gridDesci(2) = 667
       gridDesci(3) = 291
       gridDesci(4) = 23.9584
       gridDesci(5) = -124.9474
       gridDesci(7) = 50.038709
       gridDesci(8) = -65.0526214
       gridDesci(6) = 128
       gridDesci(9) = 0.0899321
       gridDesci(10) = 0.0899321
       gridDesci(20) = 64

       ALEXIobs(source)%nc = 667
       ALEXIobs(source)%nr = 291       

    elseif(ALEXIobs(source)%res.eq.4) then 
       gridDesci(1) = 0  
       gridDesci(2) = 1456
       gridDesci(3) = 625
       gridDesci(4) = 24.80
       gridDesci(5) = -125.0
       gridDesci(7) = 49.76
       gridDesci(8) = -66.8
       gridDesci(6) = 128
       gridDesci(9) = 0.04
       gridDesci(10) = 0.04
       gridDesci(20) = 64

       ALEXIobs(source)%nc = 1456
       ALEXIobs(source)%nr = 625
    else
       write(LVT_logunit,*) "[ERR] The ALEXI data resolution must be either 10 or 4"
       call LVT_endrun()
    endif
      
 
    call neighbor_interp_input(gridDesci,LVT_rc%gridDesc,&
         LVT_rc%lnc*LVT_rc%lnr, &
         ALEXIobs(source)%rlat, ALEXIobs(source)%rlon,&
         ALEXIobs(source)%n11)

    ALEXIobs(source)%startFlag = .true. 

  end subroutine ALEXI_obsinit

end module ALEXI_obsMod
