!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------------
! NASA GSFC Land surface Verification Toolkit (LVT) V1.0
!-------------------------END NOTICE -- DO NOT EDIT-----------------------------
#include "LVT_misc.h"
!BOP
! 
! !ROUTINE: readGRACEObs
! \label{readGRACEObs}
!
! !INTERFACE: 
subroutine readGRACEObs(source)
! 
! !USES:   
  use ESMF
  use LVT_coreMod,     only : LVT_rc
  use LVT_logMod
  use LVT_histDataMod
  use LVT_timeMgrMod
  use GRACE_obsMod, only : GRACEObs

#if(defined USE_NETCDF3 || defined USE_NETCDF4) 
  use netcdf
#endif

  implicit none
!
! !INPUT PARAMETERS: 
  integer, intent(in) :: source
! 
! !OUTPUT PARAMETERS:
!
! !DESCRIPTION: 
! 
!  NOTES: 
!   The GRACE output is available at monthly intervals. So 
!   the comparisons against model data should use at least a 
!   24 hour (1day) averaging interval. 
! 
! !FILES USED:
!
! !REVISION HISTORY: 
!  10 Dec 2010: Sujay Kumar, Initial Specification
! 
!EOP
  
  real        :: tws(LVT_rc%lnc, LVT_rc%lnr)
  logical     :: file_exists
  integer     :: ftn
  integer     :: timeId
  integer     :: lweId
  integer     :: tbId
  integer     :: tId
  integer     :: scaleId
  integer     :: c,r,k,t
  real        :: dt
  integer     :: currTime
  integer     :: iret
  real        :: input_data(GRACEobs(source)%nc*GRACEobs(source)%nr)
  logical*1   :: input_bitmap(GRACEobs(source)%nc*GRACEobs(source)%nr)
  real        :: output_data(LVT_rc%lnc*LVT_rc%lnr)
  logical*1   :: output_bitmap(LVT_rc%lnc*LVT_rc%lnr)
  

  tws = LVT_rc%udef

  if(GRACEobs(source)%startFlag) then 
     GRACEobs(source)%startFlag = .false. 
     
     inquire(file=trim(GRACEObs(source)%filename),exist=file_exists) 
     if(file_exists) then 
#if(defined USE_NETCDF3 || defined USE_NETCDF4) 
        write(LVT_logunit,*) '[INFO] Reading GRACE data '//&
             trim(GRACEObs(source)%filename)
        call LVT_verify(nf90_open(path=trim(GRACEObs(source)%filename),&
             mode=nf90_nowrite, ncid = ftn), &
             'nf90_open failed in readGRACEObs ')
        call LVT_verify(nf90_inq_dimid(ftn,'time',timeId),&
             'nf90_inq_dimid failed in readGRACEObs')
        call LVT_verify(nf90_inquire_dimension(ftn,timeId,&
             len=GRACEObs(source)%tdims),&
             'nf90_inq_dimension failed in readGRACEObs')
        allocate(GRACEObs(source)%tvals(GRACEObs(source)%tdims))
        allocate(GRACEObs(source)%time_bounds(2,GRACEObs(source)%tdims))
        allocate(GRACEObs(source)%lwe_thickness(GRACEObs(source)%nc, &
             GRACEObs(source)%nr,GRACEObs(source)%tdims))
        
        call LVT_verify(nf90_inq_varid(ftn,'lwe_thickness',lweId),&
             'nf90_inq_varid failed for lwe_thickness in readGRACEObs')
        call LVT_verify(nf90_get_var(ftn,lweId,GRACEObs(source)%lwe_thickness),&
             'nf90_get_var failed for lwe_thickness in readGRACEObs')
        
        call LVT_verify(nf90_inq_varid(ftn,'time_bounds',tbId),&
             'nf90_inq_varid failed for time_bounds in readGRACEObs')
        call LVT_verify(nf90_get_var(ftn,tbId,GRACEObs(source)%time_bounds),&
             'nf90_get_var failed for time_bounds in readGRACEObs')
        
        call LVT_verify(nf90_inq_varid(ftn,'time',tId),&
             'nf90_inq_varid failed for time in readGRACEObs')
        call LVT_verify(nf90_get_var(ftn,tId,GRACEObs(source)%tvals),&
             'nf90_get_var failed for time in readGRACEObs')
        
        call LVT_verify(nf90_close(ftn))
        
        if( trim(GRACEObs(source)%gracescalefile) .eq. "none" ) then
           
           write(LVT_logunit,*) '[INFO] Using unscaled GRACE data'
        else  
           write(LVT_logunit,*) '[INFO] Reading GRACE scalefactor ' &
                //trim(GRACEObs(source)%gracescalefile)
           call LVT_verify(nf90_open(path=trim(GRACEObs(source)%gracescalefile),&
                mode=nf90_nowrite, ncid = ftn), &
                'nf90_open failed in readGRACEObs scale factor')
           allocate(GRACEObs(source)%scalefactor(GRACEObs(source)%nc, &
                GRACEObs(source)%nr))
           
           call LVT_verify(nf90_inq_varid(ftn,'SCALE_FACTOR',scaleId), &
                'nf90_inq_varid failed for scale_factor in readGRACEObs')
           call LVT_verify(nf90_get_var(ftn,scaleId,GRACEObs(source)%scalefactor),&
                'nf90_get_var failed for scale_factor in readGRACEObs')   	
           call LVT_verify(nf90_close(ftn))
        end if
        
        write(LVT_logunit,*) '[INFO] Finished reading GRACE data '//&
             trim(GRACEObs(source)%filename)
        
#endif
     endif
  end if
  
  call LVT_get_julhr(LVT_rc%yr,LVT_rc%mo,LVT_rc%da,&
       LVT_rc%hr,LVT_rc%mn,LVT_rc%ss,currTime)
  
  dt =  float((currTime-GRACEobs(source)%refTime))/24.0
  
  do k=1,GRACEobs(source)%tdims
     if(floor(GRACEobs(source)%tvals(k)).eq.dt) then 
        
        input_data = LVT_rc%udef
        
        if(GRACEobs(source)%gracescalefile .eq. "none") then       
           do r=1,GRACEobs(source)%nr
              do c=1,GRACEobs(source)%nc               
                 if(GRACEobs(source)%lwe_thickness(c,r,k).ne.32767) then 
                    if(c.le.180) then 
                       input_data(c+(r-1)*GRACEobs(source)%nc+180) = & 
                            GRACEobs(source)%lwe_thickness(c,r,k)
                    else
                       input_data(c+(r-1)*GRACEobs(source)%nc-180) = & 
                            GRACEobs(source)%lwe_thickness(c,r,k)
                    endif
                 endif
              enddo
           enddo
            
        else
	   
           do r=1,GRACEobs(source)%nr
              do c=1,GRACEobs(source)%nc               
                 if(GRACEobs(source)%lwe_thickness(c,r,k).ne.32767) then 
                    if(c.le.180) then 
                       input_data(c+(r-1)*GRACEobs(source)%nc+180) = & 
                            (GRACEobs(source)%lwe_thickness(c,r,k) &
                            *GRACEobs(source)%scalefactor(c,r))
                    else
                       input_data(c+(r-1)*GRACEobs(source)%nc-180) = & 
                            (GRACEobs(source)%lwe_thickness(c,r,k) &
                            *GRACEobs(source)%scalefactor(c,r))
                    endif
                 endif
              enddo
           enddo
           
        endif
        
        input_bitmap = .false. 
        do t=1,GRACEobs(source)%nc*GRACEobs(source)%nr
           if(input_data(t).ne.LVT_rc%udef) then 
              input_bitmap(t) = .true.
           endif
        enddo
        
        call bilinear_interp(LVT_rc%gridDesc,&
             input_bitmap, input_data, &
             output_bitmap, output_data, & 
             GRACEobs(source)%nc*GRACEobs(source)%nr, & 
             LVT_rc%lnc*LVT_rc%lnr,&
             GRACEobs(source)%rlat, GRACEobs(source)%rlon, &
             GRACEobs(source)%w11, GRACEobs(source)%w12, &
             GRACEobs(source)%w21, GRACEobs(source)%w22, &
             GRACEobs(source)%n11, GRACEobs(source)%n12, &
             GRACEobs(source)%n21, GRACEobs(source)%n22, &
             LVT_rc%udef, iret)
        
        do r=1,LVT_rc%lnr
            do c=1,LVT_rc%lnc
               if(output_data(c+(r-1)*LVT_rc%lnc).ne.LVT_rc%udef) then 
                  tws(c,r) = & 
                       output_data(c+(r-1)*LVT_rc%lnc)*10.0 !to mm. 
               else
                  tws(c,r) = LVT_rc%udef
               endif
            enddo
         enddo
         exit
      endif
   enddo

   call LVT_logSingleDataStreamVar(LVT_MOC_TWS,source,&
        tws,vlevel=1,units="mm")
   
   
 end subroutine readGRACEObs

